#!/dls_sw/prod/tools/RHEL5/bin/python2.6

from pkg_resources import require
require('dls_autotestframework')
from dls_autotestframework import TestCase
#from autotestframework import *

import re
import subprocess
import time
import signal

class caAppCaseBase(TestCase):
   """
   Base class for all ca app test cases.
   """
   
   def __init__(self, A):
      TestCase.__init__(self, A)
      self.__diag = 1
      self.__timeout = 10000

   def getDiag(self):
      return self.__diag

   def getTimeout(self):
      return self.__timeout

   

class caAppCase(caAppCaseBase):
   """
   Test case to run up both test client programs simultanously.
   It also stops and starts the IOC, to check how the clients behave.
   """
   
   def runTest(self):

      #Run the first test program.
      print "Starting caApp testcaget program..."
      test1 = self.entity("testca1")
      test1.start()
      self.sleep(5)
      #Test for expected output
      self.verifyIocStdout(test1, "readAccess 05000",wait=5)
      self.verifyIocStdout(test1, "notConnected 00000",wait=5)
      
      #Run the second test program.
      print "Starting caApp testcaput program..."
      test2 = self.entity("testca2")
      test2.start()
      self.sleep(5)
      #Test for expected output
      self.verifyIocStdout(test2, "readAccess 05000 writeAccess 05000",wait=5)
      self.verifyIocStdout(test2, "connected 05000",wait=5)

      #Stop and start the IOC. This tests for 'Virtual circuit disconnect'
      ioc = self.entity("iocca")
      ioc.stop()
      #Test for output from both test programs
      self.verifyIocStderr(test1, "Virtual circuit disconnect",wait=2)
      self.verifyIocStderr(test2, "Virtual circuit disconnect",wait=2)
      self.verifyIocStdout(test1, "readAccess 00000 writeAccess 00000", wait=10)
      self.verifyIocStdout(test1, "notConnected 05000", wait=10)
      self.verifyIocStdout(test2, "readAccess 00000 writeAccess 00000", wait=10)
      ioc.start()
      self.verifyIocStdout(test1, "readAccess 05000 writeAccess 05000", wait=30)
      self.verifyIocStdout(test1, "notConnected 00000", wait=30)
      self.verifyIocStdout(test2, "readAccess 05000 writeAccess 05000", wait=30)

      time.sleep(1)

      #Send SIGTSTP. This tests for 'Virtual circuit unresponsive'
      print 'Sending SIGSTOP'
      ioc.sendSignal(signal.SIGSTOP)
      self.verifyIocStderr(test1, "Virtual circuit unresponsive", wait=40)
      self.verifyIocStdout(test1, "readAccess 05000 writeAccess 05000", wait=40)
      self.verifyIocStdout(test1, "notConnected 05000", wait=40)
      self.verifyIocStderr(test2, "Virtual circuit unresponsive", wait=40)
      self.verifyIocStdout(test2, "readAccess 05000 writeAccess 05000", wait=40)
      #Send SIGCONT
      time.sleep(5)
      print 'Sending SIGCONT'
      ioc.sendSignal(signal.SIGCONT)
      self.verifyIocStdout(test1, "notConnected 00000", wait=40)
      self.verifyIocStdout(test2, "connected 05000", wait=40)
      
      print "Done."
