#!/bin/env python2.6

import pkg_resources
pkg_resources.require('dls_autotestframework')
from dls_autotestframework import TestSuite, IocEntity, BuildEntity, TestCase, Target
pkg_resources.require('cothread')
import cothread.catools as catools
import datetime

################################################
# Test suite for dbcaPerform test
    
class dbcaPerformTestSuite(TestSuite):

    def createTests(self):
        # Define the targets for this test suite
        Target("simulation", self, entities=[\
            IocEntity('client',
                      directory="iocBoot/iocdbcaPerform",
                      bootCmd="../../bin/linux-x86/common client.main",
                      automaticRun=False),
            IocEntity('master',
                      directory="iocBoot/iocdbcaPerform",
                      bootCmd="../../bin/linux-x86/common master.main",
                      automaticRun=False),
            IocEntity('testChangeLinks',
                      directory='.',
                      bootCmd='bin/linux-x86/changeLinks 100 5 0xf',
                      automaticRun=False),
            #EpicsDbEntity('db', fileName="dbcaPerformApp/Db/master.db"),
            BuildEntity('dbcaPerformAppBuild', directory='.') ])
        #self.doBuild = True
        # The tests
        CaseTestDbcaPerform(self)
 
class CaseTestDbcaPerform(TestCase):
    def monitorCallback(self, value):
        "Monitor - expects 10 callbacks every one second and then a pause of 10 sec"        
        # print "name %s value %s" % (value.name, value)
        # capture the results in the monitor
        self.results[self.resultIndex] = value
        self.timestamps[self.resultIndex] = datetime.datetime.now()
        self.resultIndex = self.resultIndex + 1

    def runTest(self):
        print "Running dbcaPerformApp test"
        master = self.entity('master')
        master.start()
        client = self.entity('client')
        client.start()
        self.sleep(2)
        self.putPv("mrkMasterEvent.SCAN", "1 second") 
        changer = self.entity('testChangeLinks')
        #set up monitor
        self.testMonitor = catools.camonitor("mrkMasterAiFromMaster0099", self.monitorCallback)
        self.resultIndex = 0
        self.results = {}
        self.timestamps  = {}
        print "Running changeLinks for 30 seconds"
        changer.start()
        self.sleep(30)
        changer.stop()
        print "Stopping changeLinks"
        #check monitor timestamps
        #print "Results received %d" % self.resultIndex
        
        firstTime = True
        self.timeDeltas = {}
        for key, value in self.timestamps.iteritems():
            #print "Item %d: time %s" % (key, value)
            if firstTime:
                firstTime = False
            else:
                self.timeDeltas[key] = value - oldValue
            oldValue = value        

        # declare test okay if one and only one timedelta is longer than 7 seconds
        longDeltasCounts = 0
        for key,value in self.timeDeltas.iteritems():
            if (value > datetime.timedelta(seconds=7)):
                longDeltasCounts = longDeltasCounts + 1    
            print "Item %d: delta %s" % (key, value)

        if (longDeltasCounts != 1):
            self.fail("Timing tests dbcaPerform expect only one long timedelta")    
        

################################################
# Main entry point

if __name__ == "__main__":
    # Create and run the test sequence
    dbcaPerformTestSuite()

