#!/bin/env python2.6

# Test suite to use with pyUnit

from platform import node
from pkg_resources import require
require('dls_autotestframework')
from dls_autotestframework import *

################################################
# Test suite for the EPICS Snmp module
    
class TestLinearSuite(TestSuite):

    def createTests(self):
        # Define the targets for this test suite

        Target("simulation", self, entities=[\
            IocEntity('ioc',
                      directory="iocBoot/iocmisc",
                      bootCmd="../../bin/linux-x86/misc stcmd.host"),
            EpicsDbEntity('db', fileName="miscApp/Db/testLinearConvert.db"),
            BuildEntity('build', directory='.'),
            GuiEntity('medm', directory='miscApp/adl', runCmd="medm -x testLinear.adl") ])

        # The tests
        CaseTestAiOldUnipolarRaw(self)
        CaseTestAiUnipolarRaw(self)
        CaseTestAiOldBipolarRaw(self)
        CaseTestAiRoffBipolarRaw(self)
        CaseTestAiEoffBipolarRaw(self)
        CaseTestAoOldUnipolar(self)
        CaseTestAoUnipolar(self)
        CaseTestAoOldBipolar(self)
        CaseTestAoRoffBipolar(self)
        CaseTestAoEoffBipolar(self)
 
################################################
# Intermediate test case class that provides some utility functions
# for this suite

class TestLinearCase(TestCase):
    def TestInt(self, InpPV, InpMin, InpMax, OutPV, OutMin, OutMax, nSteps ):
        '''Writes 4096 values via caput to the output records and reads them back as ints via the input records'''
        for step in range(nSteps):
            i = InpMin + step * float(InpMax - InpMin) / nSteps
            self.putPv( InpPV, i )
            self.verifyPv( OutPV, OutMin + round(( i- float(InpMin)) * (OutMax - OutMin)/float(InpMax - InpMin)))

        self.putPv( InpPV, InpMin-1 )
        self.verifyPv( OutPV, OutMin )

        self.putPv( InpPV, InpMax+1 )
        self.verifyPv( OutPV, OutMax )

    def TestFloat(self, InpPV, InpMin, InpMax, OutPV, OutMin, OutMax, nSteps ):
        '''Writes 4096 values via caput to the output records and reads them back as floats via the input records'''
        eps = float(OutMax - OutMin)/(InpMax - InpMin) * 0.001

        for i in range( InpMin, InpMax+1, (InpMax-InpMin)/nSteps ):
            self.putPv( InpPV, i )
            self.verifyPvFloat( OutPV, OutMin + (float(int(i))-InpMin) * (OutMax - OutMin)/(InpMax - InpMin), eps )

        self.putPv( InpPV, InpMin-1 )
        self.verifyPv( OutPV, OutMin )

        self.putPv( InpPV, InpMax+1 )
        self.verifyPv( OutPV, OutMax )
       
        
################################################
# Test cases
    
class CaseTestAiOldUnipolarRaw(TestLinearCase):
    '''Tests Old Linear Ai Unipolar'''
    def runTest(self):
        self.TestFloat( "mrkLinearAiOldUPRaw", 0, 4095, "mrkLinearAiOldUP", 0, 10, 100 )

class CaseTestAiUnipolarRaw(TestLinearCase):
    '''Tests Linear Ai Unipolar'''
    def runTest(self):
        self.TestFloat( "mrkLinearAiUPRaw", 0, 4095, "mrkLinearAiUP", 0, 10, 100 )

class CaseTestAiOldBipolarRaw(TestLinearCase):
    '''Tests Old Linear Ai Unipolar'''
    def runTest(self):
        self.TestFloat( "mrkLinearAiOldBPRaw", -2048, 2047, "mrkLinearAiOldBP", -10, 10, 100 )

class CaseTestAiRoffBipolarRaw(TestLinearCase):
    '''Tests Linear Ai ROFF Bipolar'''
    def runTest(self):
        self.TestFloat( "mrkLinearAiroffBPRaw", -2048, 2047, "mrkLinearAiroffBP", -10, 10, 100 )

class CaseTestAiEoffBipolarRaw(TestLinearCase):
    '''Tests Linear Ai EOFF Bipolar'''
    def runTest(self):
        self.TestFloat( "mrkLinearAieoffBPRaw", -2048, 2047, "mrkLinearAieoffBP", -10, 10, 100 )

class CaseTestAoOldUnipolar(TestLinearCase):
    '''Tests Old Linear Ao Unipolar'''
    def runTest(self):
        self.TestInt( "mrkLinearAoOldUP", 0, 10, "mrkLinearAoOldUPRaw", 0, 4095, 100 )

class CaseTestAoUnipolar(TestLinearCase):
    '''Tests Linear Ao Unipolar'''
    def runTest(self):
        self.TestInt( "mrkLinearAoUP", 0, 10, "mrkLinearAoUPRaw", 0, 4095, 100 )

class CaseTestAoOldBipolar(TestLinearCase):
    '''Tests Old Linear Ao Unipolar'''
    def runTest(self):
        self.TestInt( "mrkLinearAoOldBP", -10, 10, "mrkLinearAoOldBPRaw", -2048, 2047, 100 )

class CaseTestAoRoffBipolar(TestLinearCase):
    '''Tests Linear Ao ROFF Bipolar'''
    def runTest(self):
        self.TestInt( "mrkLinearAoroffBP", -10, 10, "mrkLinearAoroffBPRaw", -2048, 2047, 100 )

class CaseTestAoEoffBipolar(TestLinearCase):
    '''Tests Linear Ao EOFF Bipolar'''
    def runTest(self):
        self.TestInt( "mrkLinearAoeoffBP", -10, 10, "mrkLinearAoeoffBPRaw", -2048, 2047, 100 )


################################################
# Main entry point

if __name__ == "__main__":
    # Create and run the test sequence
    TestLinearSuite()

    
